package com.hand.hls.dp.service.impl;

import cn.hutool.core.date.DateUtil;
import com.hand.hap.core.IRequest;
import com.hand.hap.message.IMessagePublisher;
import com.hand.hls.dp.service.SysDpDownloadLogService;
import com.hand.hls.dp.util.LogUtils;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.io.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.nio.charset.StandardCharsets;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.zip.CRC32;
import java.util.zip.CheckedOutputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * @author <a href="mailto:zhangnet14@gmail.com">Spencer Chang</a>
 * @date 2020/4/20 10:03
 * @since
 */
@Service
public class SysDpDownloadLogServiceImpl implements SysDpDownloadLogService {
    @Value("${log-path}")
    private String logPath;

    @Autowired
    private IMessagePublisher messagePublisher;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    /**
     * 通知集群服务器当天日志暂存到redis
     *
     * @param requestContext 请求上下文
     * @param request        http请求
     * @param response       http响应
     * @throws Exception 异常
     */
    @Override
    public void collectLog(IRequest requestContext, HttpServletRequest request, HttpServletResponse response) throws Exception {
        messagePublisher.publish(SysClusterLogFileSubscriber.LOGFILE_DOWNLOAD, "download logfile");
    }

    /**
     * 下载集群服务器上当天日志
     *
     * @param requestContext 请求上下文
     * @param request        http请求
     * @param response       http响应
     * @throws Exception 异常
     */
    @Override
    public void downloadClusterLogZipFile(IRequest requestContext, HttpServletRequest request, HttpServletResponse response) throws Exception {
        LocalDate tmp = LocalDate.now();
        Date date = Date.from(tmp.atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
        String zipFileName = "leaf-" + DateUtil.format(date, "yyyy-MM-dd") + ".zip";
        response.reset();
        response.setHeader("Pragma", "No-cache");
        response.setHeader("Cache-Control", "no-cache,must-revalidate");
        response.setHeader("Expires", "0");
        response.setHeader("Content-Disposition", "attachment;filename=" + new String(zipFileName.getBytes(), StandardCharsets.ISO_8859_1));
        response.setContentType("application/x-msdownload;");
        List<String> zipFileList = redisTemplate.opsForList().range(SysClusterLogFileSubscriber.LOGFILE_KEY, 0, -1);
        // 清空
        while (redisTemplate.opsForList().size(SysClusterLogFileSubscriber.LOGFILE_KEY) > 0) {
            redisTemplate.opsForList().leftPop(SysClusterLogFileSubscriber.LOGFILE_KEY);
        }
        int size = zipFileList.size();
        if (size >= 1) {
            ByteArrayOutputStream byteOutPut = new ByteArrayOutputStream();
            CheckedOutputStream cos = new CheckedOutputStream(byteOutPut, new CRC32());
            ZipOutputStream zos = new ZipOutputStream(cos);
            for (int i = 0; i < size; i++) {
                String zipFile = zipFileList.get(i);
                String singleZipFileName = "leaf-" + DateUtil.format(date, "yyyy-MM-dd") + "-" + i + ".zip";
                // 创建ZipEntry
                ZipEntry entry = new ZipEntry(singleZipFileName);
                // 放入文件，防止文件名相同的附件报错
                try {
                    zos.putNextEntry(entry);
                } catch (Exception e) {
                    // ignore
                }
                // 写入文件
                byte[] byteArray = Base64.decodeBase64(zipFile);
                ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(byteArray);
                int n = 0;
                byte[] buffer = new byte[LogUtils.DEFAULT_LARGE_BUFFER_SIZE];
                while (LogUtils.EOF != (n = byteArrayInputStream.read(buffer))) {
                    zos.write(buffer, 0, n);
                }
            }
            zos.closeEntry();
            zos.close();
            response.setHeader("Content-Length", String.valueOf(byteOutPut.toByteArray().length));
            IOUtils.copy(new ByteArrayInputStream(byteOutPut.toByteArray()), response.getOutputStream());
            cos.close();
        }
    }

    /**
     * 下载当前请求的服务器上7天日志
     *
     * @param requestContext 请求上下文
     * @param request        http请求
     * @param response       http响应
     * @throws Exception 异常
     */
    @Override
    public void download(IRequest requestContext, HttpServletRequest request, HttpServletResponse response) throws Exception {
        String zipFileName = DateUtil.format(new Date(), "yyyyMMddHHmmss") + "_logs.zip";
        response.reset();
        response.setHeader("Pragma", "No-cache");
        response.setHeader("Cache-Control", "no-cache,must-revalidate");
        response.setHeader("Expires", "0");
        response.setHeader("Content-Disposition", "attachment;filename=" + new String(zipFileName.getBytes(), StandardCharsets.ISO_8859_1));
        response.setContentType("application/x-msdownload;");
        ByteArrayOutputStream byteOutPut = new ByteArrayOutputStream();
        CheckedOutputStream cos = new CheckedOutputStream(byteOutPut, new CRC32());
        ZipOutputStream zos = new ZipOutputStream(cos);
        // 下载7天内的日志
        String[] downloadFiles = new String[7];
        int length = downloadFiles.length;
        // 当前日志
        downloadFiles[0] = logPath + "leaf.log";
        for (int i = 1; i <= length - 1; i++) {
            LocalDate tmp = LocalDate.now().minusDays(i);
            Date date = Date.from(tmp.atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
            downloadFiles[i] = logPath + "leaf-" + DateUtil.format(date, "yyyy-MM-dd") + ".log";
        }
        // 查找日志目录下所有文件
        List<String> files = new ArrayList<>(31);
        LogUtils.findFiles(new File(logPath), files);
        files.forEach(file -> {
            for (String downloadFile : downloadFiles) {
                if (file.equals(downloadFile)) {
                    // 创建ZipEntry
                    ZipEntry entry = new ZipEntry(file.substring(logPath.length()));
                    // 放入文件，防止文件名相同的附件报错
                    try {
                        zos.putNextEntry(entry);
                    } catch (Exception e) {
                        // ignore
                    }
                    // 写入文件
                    LogUtils.readFileToZip(file, zos);
                }
            }
        });
        zos.closeEntry();
        zos.close();
        response.setHeader("Content-Length", String.valueOf(byteOutPut.size()));
        IOUtils.copy(new ByteArrayInputStream(byteOutPut.toByteArray()), response.getOutputStream());
        cos.close();
    }
}
